import requests


def test_basket_subscribe(conn, create_data, extrasco_subscribe_info, reference_year):
    assert extrasco_subscribe_info['info']['controlResult']['controlOK'] is True

    def get_baskets():
        url = conn + '/get-baskets?NameID=%s' % create_data['name_id']
        resp = requests.get(url)
        resp.raise_for_status()
        assert resp.json()['err'] == 0
        return resp.json()['data']

    def subscribe(person_id):
        url = conn + '/add-person-basket-subscription?NameID=%s' % create_data['name_id']
        payload = {
            'person_id': person_id,
            'activity_id': extrasco_subscribe_info['activity']['id'],
            'unit_id': extrasco_subscribe_info['unit']['id'],
            'place_id': extrasco_subscribe_info['place']['id'],
            'start_date': extrasco_subscribe_info['unit']['dateStart'][:10],
            'end_date': extrasco_subscribe_info['unit']['dateEnd'][:10],
        }
        resp = requests.post(url, json=payload)
        resp.raise_for_status()
        return resp

    def subscriptions(person_id):
        url = conn + '/read-subscribe-activity-list?NameID=%s' % create_data['name_id']
        params = {
            'person_id': person_id,
            'nature': 'EXTRASCO',
            'school_year': '%s-%s' % (reference_year, reference_year + 1),
        }
        resp = requests.get(url, params=params)
        resp.raise_for_status()
        assert resp.json()['err'] == 0
        return resp.json()['data']

    def get_bookings(person_id):
        url = conn + '/read-activity-agenda?NameID=%s' % create_data['name_id']
        params = {
            'person_id': person_id,
            'activity_id': extrasco_subscribe_info['activity']['id'],
            'start_date': extrasco_subscribe_info['unit']['dateStart'][:10],
            'end_date': extrasco_subscribe_info['unit']['dateEnd'][:10],
        }
        resp = requests.get(url, params=params)
        resp.raise_for_status()
        assert resp.json()['err'] == 0
        return resp.json()['data']

    # no subscription
    assert subscriptions(create_data['bart_num']) == []
    assert subscriptions(create_data['maggie_num']) == []

    # empty basket
    assert get_baskets() == []

    # subscribe Bart
    resp = subscribe(create_data['bart_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert data['basket']['codeRegie'] == 105
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 1
    assert len({x['idIns'] for x in data['basket']['lignes']}) == 3

    assert len(subscriptions(create_data['bart_num'])) == 1
    assert subscriptions(create_data['maggie_num']) == []

    # basket
    data = get_baskets()
    assert len(data) == 1
    assert data[0]['codeRegie'] == 105
    assert data[0]['text'] == 'ENFANCE LOISIRS ET PE'
    assert len(data[0]['lignes']) == 3
    assert len({x['personneInfo']['numPerson'] for x in data[0]['lignes']}) == 1

    # get 3 idIns because we subscribe a generic unit
    assert len({x['idIns'] for x in data[0]['lignes']}) == 3
    basket_id = data[0]['id']

    # cannot subscribe Bart twice
    resp = subscribe(create_data['bart_num'])
    assert resp.json()['err'] == 1
    assert 'E1019' in resp.json()['err_desc']
    assert len(get_baskets()) == 1

    # delete basket
    # should be call by user or by cron job
    url = conn + '/delete-basket?NameID=%s' % create_data['name_id']
    payload = {'basket_id': basket_id}
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    assert resp.json()['data'] == 'ok'
    assert get_baskets() == []
    assert subscriptions(create_data['bart_num']) == []

    # subscribe Bart
    resp = subscribe(create_data['bart_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 1
    assert len(subscriptions(create_data['bart_num'])) == 1

    # subscribe Maggie
    resp = subscribe(create_data['maggie_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 2
    assert len(subscriptions(create_data['maggie_num'])) == 1

    # delete (generic) basket line for Bart
    data = get_baskets()
    assert len(data) == 1
    assert len(data[0]['lignes']) == 6
    basket_id = data[0]['id']
    # idIns for the generic unit
    line_id = [
        y['id']
        for x in data
        for y in x['lignes']
        if y['personneInfo']['numPerson'] == int(create_data['bart_num'])
        if y['inscription']['idUnit'] == extrasco_subscribe_info['unit']['id']
    ][0]
    url = conn + '/delete-basket-line?NameID=%s' % create_data['name_id']
    payload = {
        'basket_id': basket_id,
        'line_id': line_id,
    }
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['codeRegie'] == 105
    assert len({x['personneInfo']['numPerson'] for x in data['lignes']}) == 1
    assert len({x['idIns'] for x in data['lignes']}) == 3
    data = get_baskets()
    assert len(data) == 1
    assert len(get_baskets()) == 1
    assert len(data[0]['lignes']) == 3
    assert subscriptions(create_data['bart_num']) == []
    assert len(subscriptions(create_data['maggie_num'])) == 1

    # re-subscribe Bart
    resp = subscribe(create_data['bart_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 2
    assert len(subscriptions(create_data['bart_num'])) == 1

    # add bookings to Bart
    slots = [x['id'] for x in extrasco_subscribe_info['info']['agenda'] if x['disabled'] is False]
    url = conn + '/update-activity-agenda/?NameID=%s' % create_data['name_id']
    payload = {
        'person_id': create_data['bart_num'],
        'activity_id': extrasco_subscribe_info['activity']['id'],
        'start_date': extrasco_subscribe_info['unit']['dateStart'][:10],
        'end_date': extrasco_subscribe_info['unit']['dateEnd'][:10],
        'booking_list': [slots[0], slots[-1]],
    }
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    assert resp.json()['updated'] is True
    assert [x['booked'] for x in resp.json()['changes']] == [True, True]
    assert len([x['prefill'] for x in get_bookings(create_data['bart_num']) if x['prefill'] is True]) == 2

    # add bookings to Maggie
    slots = [':'.join([create_data['maggie_num']] + x.split(':')[1:]) for x in slots]
    url = conn + '/update-activity-agenda/?NameID=%s' % create_data['name_id']
    payload = {
        'person_id': create_data['maggie_num'],
        'activity_id': extrasco_subscribe_info['activity']['id'],
        'start_date': extrasco_subscribe_info['unit']['dateStart'][:10],
        'end_date': extrasco_subscribe_info['unit']['dateEnd'][:10],
        'booking_list': [slots[0], slots[-1]],
    }
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    assert resp.json()['updated'] is True
    assert [x['booked'] for x in resp.json()['changes']] == [True, True]
    assert len([x['prefill'] for x in get_bookings(create_data['maggie_num']) if x['prefill'] is True]) == 2

    # validate basket
    url = conn + '/validate-basket?NameID=%s' % create_data['name_id']
    payload = {'basket_id': basket_id}
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert len(data['idInsLst']) == 6
    assert len(data['factureLst']) == 0  # No invoice #2187
    assert get_baskets() == []

    assert len(subscriptions(create_data['bart_num'])) == 1
    assert len(subscriptions(create_data['maggie_num'])) == 1

    # call cancelInvoiceAndDeleteSubscribeList de remove subscriptions
