import pytest
import requests

from .conftest import diff, link, unlink


def test_direct_debit_order(conn, create_data):
    unlink(conn, create_data['name_id'])
    link(conn, create_data)

    url = conn + '/add-rl1-direct-debit-order?NameID=%s' % create_data['name_id']
    payload = {
        'codeRegie': '102',
        'bank/bankBIC': 'BDFEFR2T',
        'bank/bankIBAN': 'FR7630001007941234567890185',
        'bank/bankRUM': 'xxx',
        'bank/dateStart': '2023-01-01',
        'bank/bankAddress': '75049 PARIS cedex 01',
        'bank/civility': 'x',
        'bank/lastName': 'Ewing',
        'bank/firstName': 'John Ross',
    }
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    res = resp.json()
    assert res['data'] == 'ok'

    url = conn + '/get-rl1-direct-debit-order?NameID=%s' % create_data['name_id']
    params = {
        'codeRegie': '102',
        'dateRef': '2023-01-01',
    }
    resp = requests.get(url, params=params)
    resp.raise_for_status()
    res = resp.json()
    res['data']['numPerson'] = 'N/A'
    assert diff(res['data'], 'test_get_rl1_direct_debit_order.json')


@pytest.mark.xfail(run=False)
def test_basket_subscribe(conn, create_data, extrasco_subscribe_info, reference_year):
    assert extrasco_subscribe_info['info']['controlResult']['controlOK'] is True

    def get_baskets():
        url = conn + '/get-baskets?NameID=%s' % create_data['name_id']
        resp = requests.get(url)
        resp.raise_for_status()
        assert resp.json()['err'] == 0
        return resp.json()['data']

    def subscribe(person_id):
        url = conn + '/add-person-basket-subscription?NameID=%s' % create_data['name_id']
        payload = {
            'person_id': person_id,
            'activity_id': extrasco_subscribe_info['activity']['id'],
            'unit_id': extrasco_subscribe_info['unit']['id'],
            'place_id': extrasco_subscribe_info['place']['id'],
            'start_date': extrasco_subscribe_info['unit']['dateStart'][:10],
            'end_date': extrasco_subscribe_info['unit']['dateEnd'][:10],
        }
        resp = requests.post(url, json=payload)
        resp.raise_for_status()
        return resp

    # empty basket
    assert get_baskets() == []

    # subscribe Bart
    resp = subscribe(create_data['bart_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert data['basket']['codeRegie'] == 105
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 1
    assert len({x['idIns'] for x in data['basket']['lignes']}) == 3

    # subscribe Maggie
    resp = subscribe(create_data['maggie_num'])
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert data['controlResult'] == {'controlOK': True, 'message': None}
    assert len({x['personneInfo']['numPerson'] for x in data['basket']['lignes']}) == 2

    # basket
    data = get_baskets()
    assert len(data) == 1
    assert data[0]['codeRegie'] == 105
    assert data[0]['text'] == 'ENFANCE LOISIRS ET PE'
    assert len(data[0]['lignes']) == 3
    assert len({x['personneInfo']['numPerson'] for x in data[0]['lignes']}) == 1

    # get 3 idIns because we subscribe a generic unit
    assert len({x['idIns'] for x in data[0]['lignes']}) == 3
    basket_id = data[0]['id']

    # validate basket
    url = conn + '/validate-basket?NameID=%s' % create_data['name_id']
    payload = {'basket_id': basket_id}
    resp = requests.post(url, json=payload)
    resp.raise_for_status()
    assert resp.json()['err'] == 0
    data = resp.json()['data']
    assert len(data['idInsLst']) == 6
    assert len(data['factureLst']) == 0
    assert get_baskets() == []

    # to continue :
    # cancelInvoiceAndDeleteSubscribeList
    # payInvoice
